@@ -1,6 +1,6 @@
 /*  $Id: trans.c,v 1.19 2004/11/22 02:28:21 mgrouch Exp $  */
 
-#include "config.h"
+#include <config.h>
 #include "trans.h"
 
 /*
@@ -276,7 +276,7 @@ int xsltRun(xsltOptionsPtr ops, char* xsl, const char** params,
     xmlDocPtr doc, style;
     int i, options = 0;
 
-    options = XML_PARSE_DTDATTR;
+    options = XSLT_PARSE_OPTIONS;
      
     /*
      * Compile XSLT Sylesheet
