@@ -26,7 +26,7 @@ THE SOFTWARE.
 
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <stdio.h>
@@ -43,6 +43,8 @@ THE SOFTWARE.
 #include <libxml/parserInternals.h>
 #include <libxml/uri.h>
 
+#include "xmlstar.h"
+
 /*
    TODO:
           1. Should this be allowed ?
@@ -109,7 +111,7 @@ static XmlEdAction ops[MAX_XML_ED_OPS];
  */
 static const char edit_usage_str_1[] =
 "XMLStarlet Toolkit: Edit XML document(s)\n"
-"Usage: xml ed <global-options> {<action>} [ <xml-file-or-uri> ... ]\n"
+"Usage: %s ed <global-options> {<action>} [ <xml-file-or-uri> ... ]\n"
 "where\n"
 "  <global-options>  - global options for editing\n"
 "  <xml-file-or-uri> - input XML document file name/uri (stdin otherwise)\n\n";
@@ -152,16 +154,16 @@ static const char edit_usage_str_4[] =
  *  display short help message
  */
 void
-edUsage(int argc, char **argv)
+edUsage(const char *argv0, exit_status status)
 {
     extern const char more_info[];
-    FILE* o = stderr;
-    fprintf(o, "%s", edit_usage_str_1);
+    FILE *o = (status == EXIT_SUCCESS)? stdout : stderr;
+    fprintf(o, edit_usage_str_1, argv0);
     fprintf(o, "%s", edit_usage_str_2);
     fprintf(o, "%s", edit_usage_str_3);
     fprintf(o, "%s", edit_usage_str_4);
     fprintf(o, "%s", more_info);
-    exit(1);
+    exit(status);
 }
 
 /**
@@ -206,7 +208,7 @@ edParseOptions(edOptionsPtr ops, int argc, char **argv)
         else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h") ||
                  !strcmp(argv[i], "-?") || !strcmp(argv[i], "-Z"))
         {
-            edUsage(argc, argv);
+            edUsage(argv[0], EXIT_SUCCESS);
         }
         else
         {
@@ -244,10 +246,10 @@ edParseNSArr(const char** ns_arr, int* plen,
                 xmlChar *name, *value;
 
                 i++;
-                if (i >= count) edUsage(0, NULL);
+                if (i >= count) edUsage(argv[0], EXIT_BAD_ARGS);
 
                 for(j=0; argv[i][j] && (argv[i][j] != '='); j++);
-                if (argv[i][j] != '=') edUsage(0, NULL);
+                if (argv[i][j] != '=') edUsage(argv[0], EXIT_BAD_ARGS);
 
                 name = xmlStrndup((const xmlChar *) argv[i], j);
                 value = xmlStrdup((const xmlChar *) argv[i]+j+1);
@@ -363,7 +365,7 @@ edUpdate(xmlDocPtr doc, const char *loc, const char *val, XmlNodeType type)
                      */
 
                     /* TODO: do we need xmlEncodeEntitiesReentrant() too/instead? */
-                    xmlChar *content = xmlEncodeSpecialChars(NULL, val);
+                    xmlChar *content = xmlEncodeSpecialChars(NULL, (const xmlChar*) val);
 
                     xmlNodeSetContent(cur->nodeTab[i], content);
                     xmlFree(content);
@@ -642,6 +644,10 @@ edDelete(xmlDocPtr doc, char *str)
                 /*for (i = 0; i < cur->nodeNr; i++)*/
                 for (i = cur->nodeNr - 1; i >= 0; i--)
                 {
+                    if (cur->nodeTab[i]->type == XML_NAMESPACE_DECL) {
+                        fprintf(stderr, "FIXME: can't delete namespace nodes\n");
+                        exit(2);
+                    }
                     /*
                      *  delete node
                      */
@@ -866,6 +872,7 @@ void
 edOutput(const char* filename, edOptions g_ops)
 {
     xmlDocPtr doc = xmlParseFile(filename);
+    int format = (g_ops.noblanks && !g_ops.preserveFormat);
 
     if (!doc)
     {
@@ -880,11 +887,10 @@ edOutput(const char* filename, edOptions g_ops)
     /* Print out result */
     if (!g_ops.omit_decl)
     {
-        xmlSaveFormatFile(g_ops.inplace ? filename : "-", doc, 1);
+        xmlSaveFormatFile(g_ops.inplace ? filename : "-", doc, format);
     }
     else
     {
-        int format = 1;
         int ret = 0;
         char *encoding = NULL;
         xmlOutputBufferPtr buf = NULL;
@@ -916,8 +922,9 @@ edMain(int argc, char **argv)
     int i, j, n, start = 0;
     static edOptions g_ops;
 
-    if (argc < 3) edUsage(argc, argv);
-    if (!strcmp(argv[2], "--help") || !strcmp(argv[2], "-h")) edUsage(argc, argv);
+    if (argc < 3) edUsage(argv[0], EXIT_BAD_ARGS);
+    if (!strcmp(argv[2], "--help") || !strcmp(argv[2], "-h"))
+        edUsage(argv[0], EXIT_BAD_ARGS);
 
     edInitOptions(&g_ops);
     start = edParseOptions(&g_ops, argc, argv);
@@ -935,7 +942,7 @@ edMain(int argc, char **argv)
         if (!strcmp(argv[i], "-d") || !strcmp(argv[i], "--delete"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].op = XML_ED_DELETE;
             ops[j].arg1 = argv[i];
             ops[j].arg2 = 0;
@@ -945,11 +952,11 @@ edMain(int argc, char **argv)
         else if (!strcmp(argv[i], "-m") || !strcmp(argv[i], "--move"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].op = XML_ED_MOVE;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].arg2 = argv[i];
             ops[j].type = XML_UNDEFINED;
             j++;
@@ -957,19 +964,20 @@ edMain(int argc, char **argv)
         else if (!strcmp(argv[i], "-u") || !strcmp(argv[i], "--update"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].type = XML_UNDEFINED;
             ops[j].op = XML_ED_UPDATE;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value") &&
-                strcmp(argv[i], "-x") && strcmp(argv[i], "--expr")) edUsage(argc, argv);
+                strcmp(argv[i], "-x") && strcmp(argv[i], "--expr"))
+                edUsage(argv[0], EXIT_BAD_ARGS);
 
             if (!strcmp(argv[i], "-x") || !strcmp(argv[i], "--expr"))
                 ops[j].type = XML_EXPR;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].arg2 = argv[i];
 
             j++;
@@ -977,30 +985,31 @@ edMain(int argc, char **argv)
         else if (!strcmp(argv[i], "-r") || !strcmp(argv[i], "--rename"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].type = XML_UNDEFINED;
             ops[j].op = XML_ED_RENAME;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
+            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value"))
+                edUsage(argv[0], EXIT_BAD_ARGS);
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].arg2 = argv[i];
             j++;
         }
         else if (!strcmp(argv[i], "-i") || !strcmp(argv[i], "--insert"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);
             ops[j].type = XML_UNDEFINED;
             ops[j].op = XML_ED_INSERT;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             if (!strcmp(argv[i], "elem"))
             {
                 ops[j].type = XML_ELEM;
@@ -1013,33 +1022,33 @@ edMain(int argc, char **argv)
             {
                 ops[j].type = XML_TEXT;
             }
-            else edUsage(argc, argv);
+            else edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg3 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg2 = argv[i];
             j++;
         }
         else if (!strcmp(argv[i], "-a") || !strcmp(argv[i], "--append"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].type = XML_UNDEFINED;
             ops[j].op = XML_ED_APPEND;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             if (!strcmp(argv[i], "elem"))
             {
                 ops[j].type = XML_ELEM;
@@ -1052,33 +1061,33 @@ edMain(int argc, char **argv)
             {
                 ops[j].type = XML_TEXT;
             }
-            else edUsage(argc, argv);
+            else edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg3 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg2 = argv[i];
             j++;
         }
         else if (!strcmp(argv[i], "-s") || !strcmp(argv[i], "--subnode"))
         {
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].type = XML_UNDEFINED;
             ops[j].op = XML_ED_SUBNODE;
             ops[j].arg1 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-t") && strcmp(argv[i], "--type")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             if (!strcmp(argv[i], "elem"))
             {
                 ops[j].type = XML_ELEM;
@@ -1091,18 +1100,18 @@ edMain(int argc, char **argv)
             {
                 ops[j].type = XML_TEXT;
             }
-            else edUsage(argc, argv);
+            else edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-n") && strcmp(argv[i], "--name")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg3 = argv[i];
             i++;
-            if (i >= argc) edUsage(argc, argv);
-            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
+            if (strcmp(argv[i], "-v") && strcmp(argv[i], "--value")) edUsage(argv[0], EXIT_BAD_ARGS);;
             i++;
-            if (i >= argc) edUsage(argc, argv);
+            if (i >= argc) edUsage(argv[0], EXIT_BAD_ARGS);;
             ops[j].arg2 = argv[i];
             j++;
         }
