@@ -26,7 +26,7 @@ THE SOFTWARE.
 
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,11 +40,29 @@ THE SOFTWARE.
 #include <libxml/xmlmemory.h>
 #include <libxml/c14n.h>
 
+#include "xmlstar.h"
 #include "escape.h"
 
+#if !HAVE_LSTAT
+# if HAVE_STAT
+#  define lstat stat
+# else
+/* TODO: #ifdef out code that uses stat instead */
+#  error "lstat() or stat() required"
+# endif
+#endif
+
+#ifndef S_ISLNK
+# define S_ISLNK(m) 0
+#endif
+
+#ifndef S_ISSOCK
+# define S_ISSOCK(m) 0
+#endif
+
 static const char ls_usage_str[] =
 "XMLStarlet Toolkit: List directory as XML\n"
-"Usage: xml ls\n"
+"Usage: %s ls\n"
 "Lists current directory in XML format.\n"
 "Time is shown per ISO 8601 spec.\n"
 "\n";
@@ -53,43 +71,27 @@ static const char ls_usage_str[] =
  *  Print small help for command line options
  */
 void
-lsUsage(int argc, char **argv)
+lsUsage(int argc, char **argv, exit_status status)
 {
     extern const char more_info[];
-    FILE* o = stderr;
-    fprintf(o, "%s", ls_usage_str);
+    FILE *o = (status == EXIT_SUCCESS)? stdout : stderr;
+    fprintf(o, ls_usage_str, argv[0]);
     fprintf(o, "%s", more_info);
-    exit(1);
+    exit(status);
 }
 
 
 char *
 get_file_type(mode_t mode)
 {
-   switch(mode & S_IFMT)
-   {
-       case S_IFREG:
-            return("f"); /* regular file */
-       case S_IFDIR:
-            return("d"); /* directory */
-       case S_IFCHR:
-            return("c"); /* character device */
-       case S_IFBLK:
-            return("b"); /* block device */
-#ifdef S_IFLNK
-       case S_IFLNK:
-            return("l"); /* symbolic link */
-#endif
-       case S_IFIFO:
-            return("p"); /* fifo */
-#ifdef S_IFSOCK
-       case S_IFSOCK:
-            return("s"); /* socket */
-#endif
-       default:
-            return("u"); /* unknown */
-   }
-   return(NULL);
+    if (S_ISREG(mode)) return "f";       /* regular file */
+    else if (S_ISDIR(mode)) return "d";  /* directory */
+    else if (S_ISCHR(mode)) return "c";  /* character device */
+    else if (S_ISBLK(mode)) return "b";  /* block device */
+    else if (S_ISLNK(mode)) return "l";  /* symlink */
+    else if (S_ISFIFO(mode)) return "p"; /* fifo */
+    else if (S_ISSOCK(mode)) return "s"; /* socket */
+    else return "u";                     /* unknown */
 }
 
 char *
@@ -104,15 +106,15 @@ get_file_perms(mode_t mode)
 
    for(i=0; i<3; i++)
    {
-       if(mode &(S_IREAD>>(i*3)))
+       if(mode &(S_IRUSR>>(i*3)))
            *p='r';
        ++p;
 
-       if(mode &(S_IWRITE>>(i*3)))
+       if(mode &(S_IWUSR>>(i*3)))
            *p='w';
        ++p;
 
-       if(mode &(S_IEXEC>>(i*3)))
+       if(mode &(S_IXUSR>>(i*3)))
            *p='x';
        ++p;
    }
@@ -160,12 +162,8 @@ xml_print_dir(char* dir)
 
       if ((d->d_name == NULL) || !strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
           continue;
-      
-#if defined (__MINGW32__)
-      if(stat(d->d_name, &stats) != 0)
-#else
+
       if(lstat(d->d_name, &stats) != 0)
-#endif
       {
         fprintf(stderr, "couldn't stat: %s\n", d->d_name);
       }
@@ -205,7 +203,7 @@ int
 lsMain(int argc, char** argv)
 {
     int res = -1;
-    if (argc != 2) lsUsage(argc, argv);
+    if (argc != 2) lsUsage(argc, argv, EXIT_BAD_ARGS);
     printf("<dir>\n");
     res = xml_print_dir(".");
     printf("</dir>\n");
