@@ -26,12 +26,13 @@ THE SOFTWARE.
 
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "xmlstar.h"
 #include "trans.h"
 #include "stack.h"
 
@@ -65,7 +66,7 @@ typedef selOptions *selOptionsPtr;
  */
 static const char select_usage_str_1[] =
 "XMLStarlet Toolkit: Select from XML document(s)\n"
-"Usage: xml sel <global-options> {<template>} [ <xml-file> ... ]\n"
+"Usage: %s sel <global-options> {<template>} [ <xml-file> ... ]\n"
 "where\n"
 "  <global-options> - global options for selecting\n"
 "  <xml-file> - input XML document file name/uri (stdin is used if missing)\n"
@@ -152,12 +153,12 @@ static const char select_usage_str_9[] =
  *  Print small help for command line options
  */
 void
-selUsage(int argc, char **argv)
+selUsage(const char *argv0, exit_status status)
 {
     extern const char more_info[];
     extern const char libxslt_more_info[];
-    FILE* o = stderr;
-    fprintf(o, "%s", select_usage_str_1);
+    FILE *o = (status == EXIT_SUCCESS)? stdout : stderr;
+    fprintf(o, select_usage_str_1, argv0);
     fprintf(o, "%s", select_usage_str_2);
     fprintf(o, "%s", select_usage_str_3);
     fprintf(o, "%s", select_usage_str_4);
@@ -168,7 +169,7 @@ selUsage(int argc, char **argv)
     fprintf(o, "%s", select_usage_str_9);
     fprintf(o, "%s", more_info);
     fprintf(o, "%s", libxslt_more_info);
-    exit(1);
+    exit(status);
 }
 
 /**
@@ -209,10 +210,10 @@ selParseNSArr(const char** ns_arr, int* plen,
                 xmlChar *name, *value;
 
                 i++;
-                if (i >= count) selUsage(0, NULL);
+                if (i >= count) selUsage(argv[0], EXIT_BAD_ARGS);
 
                 for(j=0; argv[i][j] && (argv[i][j] != '='); j++);
-                if (argv[i][j] != '=') selUsage(0, NULL);
+                if (argv[i][j] != '=') selUsage(argv[0], EXIT_BAD_ARGS);
 
                 name = xmlStrndup((const xmlChar *) argv[i], j);
                 value = xmlStrdup((const xmlChar *) argv[i]+j+1);
@@ -317,7 +318,7 @@ selParseOptions(selOptionsPtr ops, int argc, char **argv)
         else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h") ||
                  !strcmp(argv[i], "-?") || !strcmp(argv[i], "-Z"))
         {
-            selUsage(argc, argv);
+            selUsage(argv[0], EXIT_SUCCESS);
         }
         i++;
     }
@@ -368,13 +369,18 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
     {
         if(!strcmp(argv[i], "-c") || !strcmp(argv[i], "--copy-of"))
         {
+            xmlChar *xmlText;
             templateEmpty = 0;
             for (j=0; j <= m; j++) c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "  ");
-            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "<xsl:copy-of select=\"%s\"/>\n", argv[i+1]);
+            xmlText = xmlEncodeSpecialChars(NULL, BAD_CAST argv[i+1]);
+            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1,
+                "<xsl:copy-of select=\"%s\"/>\n", xmlText);
+            xmlFree(xmlText);
             i++;
         }
         else if(!strcmp(argv[i], "-v") || !strcmp(argv[i], "--value-of"))
         {
+            xmlChar *xmlText;
             templateEmpty = 0;
             if ((i+1) >= argc)
             {
@@ -383,11 +389,15 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
                 exit (1);
             }
             for (j=0; j <= m; j++) c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "  ");
-            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "<xsl:value-of select=\"%s\"/>\n", argv[i+1]);
+            xmlText = xmlEncodeSpecialChars(NULL, BAD_CAST argv[i+1]);
+            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1,
+                "<xsl:value-of select=\"%s\"/>\n", xmlText);
+            xmlFree(xmlText);
             i++;
         }
         else if(!strcmp(argv[i], "-o") || !strcmp(argv[i], "--output"))
         {
+            xmlChar *xmlText;
             templateEmpty = 0;
             if ((i+1) >= argc)
             {
@@ -396,8 +406,10 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
                 exit (1);
             }
             for (j=0; j <= m; j++) c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "  ");
+            xmlText = xmlEncodeSpecialChars(NULL, BAD_CAST argv[i+1]);
             c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1,
-                "<xsl:text><![CDATA[%s]]></xsl:text>\n", argv[i+1]);
+                "<xsl:text>%s</xsl:text>\n", xmlText);
+            xmlFree(xmlText);
             i++;
         }
         else if(!strcmp(argv[i], "-f") || !strcmp(argv[i], "--inp-name"))
@@ -414,6 +426,7 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
         }
         else if(!strcmp(argv[i], "-i") || !strcmp(argv[i], "--if"))
         {
+            xmlChar *xmlText;
             templateEmpty = 0;
             if ((i+1) >= argc)
             {
@@ -422,7 +435,10 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
                 exit (1);
             }
             for (j=0; j <= m; j++) c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "  ");
-            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "<xsl:if test=\"%s\">\n", argv[i+1]);
+            xmlText = xmlEncodeSpecialChars(NULL, BAD_CAST argv[i+1]);
+            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1,
+                "<xsl:if test=\"%s\">\n", xmlText);
+            xmlFree(xmlText);
             stack_push(stack, STK_IF);
             m++;
             i++;
@@ -459,6 +475,7 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
         }
         else if(!strcmp(argv[i], "-m") || !strcmp(argv[i], "--match"))
         {
+            xmlChar *xmlText;
             templateEmpty = 0;
             if ((i+1) >= argc)
             {
@@ -467,7 +484,10 @@ selGenTemplate(char* xsl_buf, int *len, selOptionsPtr ops, int num,
                 exit (1);
             }
             for (j=0; j <= m; j++) c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "  ");
-            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1, "<xsl:for-each select=\"%s\">\n", argv[i+1]);
+            xmlText = xmlEncodeSpecialChars(NULL, BAD_CAST argv[i+1]);
+            c += snprintf(xsl_buf + c, MAX_XSL_BUF - c - 1,
+                "<xsl:for-each select=\"%s\">\n", xmlText);
+            xmlFree(xmlText);
             stack_push(stack, STK_MATCH);
             m++;
             i++;
@@ -749,11 +769,11 @@ selMain(int argc, char **argv)
     static selOptions ops;
     static const char *params[2 * MAX_PARAMETERS + 1];
     static const char *ns_arr[2 * MAX_NS_ARGS + 1];
-    int start, c, i, n;
+    int start, c, i, n, status = 0;
     int nCount = 0;
     int nbparams;
 
-    if (argc <= 2) selUsage(argc, argv);
+    if (argc <= 2) selUsage(argv[0], EXIT_BAD_ARGS);
 
     selInitOptions(&ops);
     xsltInitOptions(&xsltOps);
@@ -799,6 +819,8 @@ selMain(int argc, char **argv)
             doc = xmlParseFile(argv[n]);
             if (doc != NULL) {
                 xsltProcess(&xsltOps, doc, params, cur, argv[n]);
+            } else {
+                status = 2;
             }
             xsltFreeStylesheet(cur);
         }
@@ -818,6 +840,8 @@ selMain(int argc, char **argv)
         doc = xmlParseFile("-");
         if (doc != NULL) {
             xsltProcess(&xsltOps, doc, params, cur, "-");
+        } else {
+            status = 2;
         }
         xsltFreeStylesheet(cur);
     }
@@ -828,6 +852,6 @@ selMain(int argc, char **argv)
     xsltCleanupGlobals();
     xmlCleanupParser();
     
-    return 0;
+    return status;
 }
 
